@@ -4,19 +4,20 @@ Created on 28 Oct 2021
 @author: jacklok
 '''
 
-from flask import Blueprint 
-import logging
+from flask import Blueprint, request, url_for, jsonify
+import logging, json
 from trexapi.decorators.api_decorators import auth_token_required
 from trexlib.utils.string_util import is_not_empty, is_empty
 from trexadmin.libs.http import create_rest_message
 from trexadmin.libs.http import StatusCode
 from trexmodel.models.datastore.pos_models import POSSetting
 from trexmodel.utils.model.model_util import create_db_client
-from flask.globals import request
-from flask.helpers import url_for
 from trexmodel.models.datastore.product_models import ProductCatalogue,\
     ProductCategory
 from trexlib.utils.common.common_util import sort_list
+from firebase_admin import firestore
+from datetime import datetime
+
 
 #logger = logging.getLogger('api')
 logger = logging.getLogger('debug')
@@ -28,6 +29,7 @@ pos_api_bp = Blueprint('pos_api_bp', __name__,
 
 @pos_api_bp.route('/check-activation', methods=['POST'])
 def check_activation():
+    
     activation_code = request.args.get('activation_code') or request.form.get('activation_code') or request.json.get('activation_code')
     logger.debug('activation_code=%s', activation_code)
     
@@ -87,13 +89,77 @@ def getPOSAccountSetting(activation_code):
         return create_rest_message('Activation code is required', status_code=StatusCode.BAD_REQUEST)
     
     
-@pos_api_bp.route('/activate', methods=['POST'])
+def updateActivationAndGetPOSAccountSetting(activation_code, device_id):
+    if is_not_empty(activation_code) and is_not_empty(device_id):
+        db_client = create_db_client(caller_info="updateActivationAndGetPOSAccountSetting")
+        with db_client.context():
+            pos_setting = POSSetting.get_by_activation_code(activation_code)
+        
+        if pos_setting:
+            logger.info('POS activation code is valid');
+            
+            is_valid = False
+            pos_setting_details = None
+            
+            with db_client.context():
+                pos_setting = POSSetting.get_by_activation_code(activation_code)
+                if pos_setting.activated:
+                    if pos_setting.device_id == device_id:
+                        is_valid = True
+                else:
+                    is_valid = True
+                    pos_setting.activate(device_id)
+            
+                if is_valid:    
+                    pos_setting_details                                 = pos_setting.setting
+                
+                    pos_setting_details['logo_image_url']               = url_for('system_bp.merchant_logo_image_url', merchant_act_key=pos_setting_details.get('account_id'))
+            
+            if is_valid:
+                
+                logger.debug('pos_setting_details=%s', pos_setting_details);
+                
+                return create_rest_message(status_code=StatusCode.OK,
+                                           **pos_setting_details
+                                           )
+            else:
+                if pos_setting.activated and pos_setting.device_id != device_id:
+                    return create_rest_message('The code have been used to activate before', status_code=StatusCode.BAD_REQUEST)
+                else:
+                    return create_rest_message('Failed to activate', status_code=StatusCode.BAD_REQUEST)
+            
+        else:
+            logger.info('POS setting record is not found');
+            return create_rest_message('Invalid activate code', status_code=StatusCode.BAD_REQUEST)
+    else:
+        logger.info('activation_code is empty or device id is empty');
+        return create_rest_message('Activation code and device id are required', status_code=StatusCode.BAD_REQUEST)
+        
+    
 @pos_api_bp.route('/account-sync', methods=['get'])
 def activate():
-    activation_code = request.args.get('activation_code') or request.form.get('activation_code') or request.json.get('activation_code')
+    try:
+        activation_code = request.args.get('activation_code') or request.form.get('activation_code') or request.json.get('activation_code')
+        logger.debug('activation_code=%s', activation_code)
+        return getPOSAccountSetting(activation_code)
+    except:
+        logger.error('Missing activation code')
+        return create_rest_message('Missing activate code', status_code=StatusCode.BAD_REQUEST)
     
-    logger.debug('activation_code=%s', activation_code)
-    return getPOSAccountSetting(activation_code)
+@pos_api_bp.route('/activate', methods=['POST'])
+def activate_post():
+    try:
+        activation_code = request.args.get('activation_code') or request.form.get('activation_code') or request.json.get('activation_code')
+        device_id       = request.args.get('device_id') or request.form.get('device_id') or request.json.get('device_id')
+        
+        logger.debug('activation_code=%s', activation_code)
+        logger.debug('device_id=%s', device_id)
+        
+        return updateActivationAndGetPOSAccountSetting(activation_code, device_id)
+    
+    except:
+        logger.error('Missing activation code')
+        return create_rest_message('Missing activate code', status_code=StatusCode.BAD_REQUEST)    
        
     
 def parse_category_to_json(category_tree_structure):
@@ -213,7 +279,7 @@ def check_catalogue_status():
         return create_rest_message('Activation code is required', status_code=StatusCode.BAD_REQUEST)
     
 @pos_api_bp.route('/get-catalogue', methods=['GET'])
-@auth_token_required
+#@auth_token_required
 def get_catalogue():
     activation_code = request.args.get('activation_code') or request.form.get('activation_code') or request.json.get('activation_code')
     
@@ -247,27 +313,11 @@ def get_catalogue():
                         last_updated_datetime = product_catalogue.modified_datetime
                         if assigned_outlet.modified_datetime is not None and assigned_outlet.modified_datetime>last_updated_datetime:
                             last_updated_datetime = assigned_outlet.modified_datetime
-                        '''
-                        dinning_option_json = []
-                        dinning_option_list = DinningOption.list_by_merchant_acct(merchant_acct)
-                        
-                        
-                        if dinning_option_list:
-                            for d in dinning_option_list:
-                                dinning_option_json.append({
-                                                            'option_key'                : d.key_in_str,
-                                                            'option_name'               : d.name,
-                                                            'is_default'                : d.is_default,
-                                                            'dinning_table_is_required' : d.dinning_table_is_required,
-                                                            'assign_queue'              : d.assign_queue,
-                                                            })
-                        '''        
                         valid_return =  {
-                                                'category'              : category_tree_structure_in_json,
-                                                'menu'                  : product_catalogue.published_menu_settings,
-                                                #'dinning_table_list'    : assigned_outlet.assigned_dinning_table_list,
-                                                #'dinning_option_list'   : dinning_option_json,
-                                                'last_updated_datetime' : last_updated_datetime.strftime('%d-%m-%Y %H:%M:%S')
+                                                'key'                       : catalogue_key,    
+                                                'category_list'             : category_tree_structure_in_json,
+                                                'product_by_category_map'   : product_catalogue.published_menu_settings,
+                                                'last_updated_datetime'     : last_updated_datetime.strftime('%d-%m-%Y %H:%M:%S')
                                             } 
                     
                 
@@ -312,16 +362,12 @@ def version_sync():
                                                 
                                             }
                                         ]
-                                '''        
-        
+                                        
+                                '''
                                 'cart'  :[
-                                            {
-                                                "table_name": "cart",
-                                                "version" : 1,
-                                                "script": "ALTER TABLE invoice ADD COLUMN void_by TEXT",
-                                                
-                                            }
+                                            
                                         ]
+                                       
                             }
                                 
     
@@ -329,6 +375,99 @@ def version_sync():
     
     return version
 
-
-     
+@pos_api_bp.route('/sales-overview/merchant/<merchant_code>/outlet/<outlet_key>/date/<transact_date>', methods=['GET'])
+#@auth_token_required
+def sales_overview_stat(merchant_code, outlet_key, transact_date):
+    total_sales_amount              = .0
+    total_tax_amount                = .0
+    total_service_charge_amount     = .0
+    total_paid_count                = 0
+    total_voided_count              = 0
+    total_item_sold_count           = 0
+    
+    firebase        = firestore.client()        
+    invoice_path    = 'merchant/%s/outlet/%s/date/%s/invoice' % (merchant_code, outlet_key, transact_date)
+    
+    logger.debug('invoice_path=%s', invoice_path)
+    
+    invoice_list_ref = firebase.collection(invoice_path).stream()
+    #invoice_list_ref = firebase.collection('merchant').document(merchant_code).collection('outlet').document(outlet_key).collection('date').document(transact_date).collection('invoice').stream()
+    
+    
+    try:
+        for invoice_ref in invoice_list_ref:
+            invoice = invoice_ref.to_dict()
+            logger.debug('invoice=%s', invoice)
+            
+            if invoice.get('is_void') == 1:
+                total_voided_count+=1
+                
+            elif invoice.get('is_paid') == 1:
+                payment_details     = json.loads(invoice.get('payment_details'))
+                transaction_items   = json.loads(invoice.get('transaction_items'))
+                total_sales_amount+= payment_details.get('net_total_amount')
+                total_paid_count+=1
+                
+                for tax_details in payment_details.get('tax_details_list'):
+                    total_tax_amount+=tax_details.get('tax_amount')
+                    
+                for service_charge_details in payment_details.get('service_charge_details_list'):
+                    total_service_charge_amount+=service_charge_details.get('charge_amount')
+                
+                for item in transaction_items:
+                    total_item_sold_count+=item.get('ordered_quantity')
+                
+                
+            
+                
+    except Exception as e:
+        logger.error('%s', e)
+    
+    return jsonify({
+                    'total_paid_count'              : total_paid_count,
+                    'total_voided_count'            : total_voided_count,
+                    'total_sales_amount'            : total_sales_amount,
+                    'total_tax_amount'              : total_tax_amount,
+                    'total_service_charge_amount'   : total_service_charge_amount,
+                    'total_item_sold_count'         : total_item_sold_count,
+                    })
+    
+  
+#@pos_api_bp.route('/counter/<date_value>', methods=['POST'])
+#@auth_token_required
+@pos_api_bp.route('/counter/merchant/<merchant_code>/outlet/<outlet_key>/date/<transact_date>', methods=['POST'])
+def counter_post(merchant_code, outlet_key, transact_date):
+    value = request.args.get('value') or request.form.get('value') or request.json.get('value')
+    firebase = firestore.client()
            
+    #counter_ref = firebase.collection('test').document('counter').collection(date_value).document('details')
+    counter_ref = firebase.collection('merchant').document(merchant_code).collection('outlet').document(str(outlet_key)).collection('date').document(transact_date).collection('counter').document('details')
+    
+    if counter_ref is None:
+        logger.debug('Counter is not found')
+        counter_ref.set({'value':value})
+    else:    
+        logger.debug('Counter is found')
+        counter_ref.set({'value':value})
+    
+    return 'OK'
+
+#@pos_api_bp.route('/counter/<date_value>', methods=['GET'])
+@pos_api_bp.route('/counter/merchant/<merchant_code>/outlet/<outlet_key>/date/<transact_date>', methods=['GET'])
+#@auth_token_required
+def counter_get(merchant_code, outlet_key, transact_date):
+    firebase = firestore.client()
+     
+    counter_ref = firebase.collection('merchant').document(merchant_code).collection('outlet').document(str(outlet_key)).collection('date').document(transact_date).collection('counter').document('details')
+    #counter_ref = firebase.collection('test').document('counter').collection(date_value).document('details')
+    counter_value = 0
+    if counter_ref is not None:
+        counter_value = counter_ref.get().get('value')
+    
+    return jsonify({
+                    'value': counter_value,
+                })
+
+
+
+ 
\ No newline at end of file
